/*********************************************************************

    SCRIPT:             features1.c
    LAST_MODIFICATION:  04/14/2009
    AUTHOR:             Christophe Nicolas Magnan
    CONTACT:            cmagnan@ics.uci.edu

    SHORT DESCRIPTION:  This script computes the features for
                        the first step of SOLpro

*********************************************************************/

/** Preprocessing **/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#define SIZE_SEQ 2010
#define SIZE_PATH 300

/** Global Variables: Files and script parameters **/
char FileIN[SIZE_PATH];			// Script parameter
char FileOUT[SIZE_PATH];		// Current output file
FILE * in, * out;			// File descriptors

/** Global Variables: Protein Features **/
char SEQ[SIZE_SEQ];			// Primary sequence
char SSS[SIZE_SEQ];			// Predicted secondary sequence
char ACC[SIZE_SEQ];			// Predicted solvent accessibility
int Len;				// Sequence length
int NbDom;				// Number of predicted domains
double TurnForm;			// Turn forming residues fraction
double AbsCharge;			// Absolute charge per residue
double MolWeight;			// Molecular weight
double Gravy;				// Grand Average of Hydropathy
double AliIndex;			// Aliphatic index
double BetaR;				// Beta residues fraction
double AlphaR;				// Alpha residues fraction
double ExpR;				// Exposed residues fraction
double MF[7][20];			// Mono-peptide frequencies
double DF[7][400];			// Di-peptide frequencies
double TF[7][8000];			// Tri-peptide frequencies

/** Functions included at the end of this script **/
void Initialize_Protein_Features();
void Check_Script_Parameter();
void Read_Input_File();
void Compute_Turn_Forming();
void Compute_Abs_Charge();
void Compute_Mol_Weight();
void Compute_Gravy();
void Compute_Aliph_Index();
void Compute_Beta_Ratio();
void Compute_Alpha_Ratio();
void Compute_Exposed_Ratio();
void Compute_All_Frequencies();
void Print_Features(char * suff, int set);

/** Main function **/
int main(int argc, char *argv[])
{
  /** Statistics Initialization **/
  Initialize_Protein_Features();
  
  /** Script parameters **/
  if(argc!=2){ exit(0); }
  strcpy(FileIN,argv[1]);
  Check_Script_Parameter();
  
  /** Input file parsing **/
  Read_Input_File();
  
  /** Feature set 1 **/
  Compute_Turn_Forming();   Compute_Abs_Charge();
  Compute_Mol_Weight();     Compute_Gravy();
  Compute_Aliph_Index();    Print_Features("01",1);
  
  /** Feature set 2 **/
  Compute_Beta_Ratio();     Compute_Alpha_Ratio();
  Compute_Exposed_Ratio();  Print_Features("02",2);
  
  /** Feature sets 3 to 20 **/
  Compute_All_Frequencies();
  Print_Features("03",3);  Print_Features("04",4);
  Print_Features("05",5);  Print_Features("06",6);
  Print_Features("07",7);  Print_Features("08",8);
  Print_Features("09",9);  Print_Features("10",10);
  Print_Features("11",11); Print_Features("12",12);
  Print_Features("13",13); Print_Features("14",14);
  Print_Features("15",15); Print_Features("16",16);
  Print_Features("17",17); Print_Features("18",18);
  Print_Features("19",19); Print_Features("20",20);
  
  /** End of the script **/
  return 0;
}

/** Functions **/
void Clean_Array(char * a,int b,int l)
{
  int i; for(i=b;i<l;i++){ a[i]='\0'; }
}

int Is_Sep(char c)
{
  if((c==' ')||(c=='\n')||(c=='\r')||(c=='\t')){ return 1; }
  return 0;
}

int Cluster_Size(int clus)
{
  if(clus==0){ return 20; }
  if(clus==1){ return 5; }
  if(clus==2){ return 7; }
  if(clus==3){ return 8; }
  if(clus==4){ return 14; }
  if(clus==5){ return 17; }
  if(clus==6){ return 7; }
  return -1;
}

void Print_Features(char * suff, int set)
{
  int i,s;
  sprintf(FileOUT,"%s%s",FileIN,suff);
  if((out=fopen(FileOUT,"w"))==NULL){ exit(0); }
  fprintf(out,"0 ");
  switch(set)
  {
    case 1:  fprintf(out,"1:%d 2:%.6f ",Len,TurnForm);
             fprintf(out,"3:%.6f 4:%.6f ",AbsCharge,MolWeight);
             fprintf(out,"5:%.6f 6:%.6f\n",Gravy,AliIndex); break;
    case 2:  fprintf(out,"1:%.6f 2:%.6f ",BetaR,AlphaR);
             fprintf(out,"3:%.6f 4:%d\n",ExpR,NbDom); break;
    case 3:  s=Cluster_Size(0);
             for(i=0;i<s;i++)
             { if(MF[0][i]!=0){ fprintf(out,"%d:%.6f ",i+1,MF[0][i]); } }
             fprintf(out,"\n"); break;
    case 4:  fprintf(out,"1:%.6f 2:%.6f ",DF[0][24],DF[0][34]);
             fprintf(out,"3:%.6f 4:%.6f ",DF[0][49],DF[0][101]);
             fprintf(out,"5:%.6f 6:%.6f ",DF[0][180],DF[0][190]);
             fprintf(out,"7:%.6f 8:%.6f ",DF[0][202],DF[0][207]);
             fprintf(out,"9:%.6f 10:%.6f ",DF[0][245],DF[0][290]);
             fprintf(out,"11:%.6f 12:%.6f ",DF[0][307],DF[0][349]);
             fprintf(out,"13:%.6f\n",DF[0][366]); break;
    case 5:  fprintf(out,"1:%.6f 2:%.6f ",TF[0][44],TF[0][146]);
             fprintf(out,"3:%.6f 4:%.6f ",TF[0][165],TF[0][305]);
             fprintf(out,"5:%.6f 6:%.6f ",TF[0][2263],TF[0][2287]);
             fprintf(out,"7:%.6f 8:%.6f ",TF[0][2296],TF[0][2922]);
             fprintf(out,"9:%.6f 10:%.6f ",TF[0][3145],TF[0][3440]);
             fprintf(out,"11:%.6f 12:%.6f ",TF[0][3734],TF[0][3860]);
             fprintf(out,"13:%.6f 14:%.6f ",TF[0][4407],TF[0][5925]);
             fprintf(out,"15:%.6f 16:%.6f ",TF[0][6049],TF[0][6108]);
             fprintf(out,"17:%.6f 18:%.6f ",TF[0][6114],TF[0][6514]);
             fprintf(out,"19:%.6f 20:%.6f ",TF[0][6780],TF[0][6867]);
             fprintf(out,"21:%.6f 22:%.6f ",TF[0][6957],TF[0][6995]);
             fprintf(out,"23:%.6f 24:%.6f\n",TF[0][7147],TF[0][7602]); break;
    case 6:  s=Cluster_Size(1);
             for(i=0;i<s;i++)
             { if(MF[1][i]!=0){ fprintf(out,"%d:%.6f ",i+1,MF[1][i]); } }
             fprintf(out,"\n"); break;
    case 7:  fprintf(out,"1:%.6f 2:%.6f ",TF[1][10],TF[1][16]);
             fprintf(out,"3:%.6f 4:%.6f ",TF[1][18],TF[1][30]);
             fprintf(out,"5:%.6f 6:%.6f ",TF[1][31],TF[1][41]);
             fprintf(out,"7:%.6f 8:%.6f ",TF[1][49],TF[1][61]);
             fprintf(out,"9:%.6f 10:%.6f ",TF[1][68],TF[1][87]);
             fprintf(out,"11:%.6f 12:%.6f\n",TF[1][106],TF[1][115]); break;
    case 8:  s=Cluster_Size(2);
             for(i=0;i<s;i++)
             { if(MF[2][i]!=0){ fprintf(out,"%d:%.6f ",i+1,MF[2][i]); } }
             fprintf(out,"\n"); break;
    case 9:  fprintf(out,"1:%.6f 2:%.6f ",DF[2][0],DF[2][2]);
             fprintf(out,"3:%.6f 4:%.6f ",DF[2][4],DF[2][5]);
             fprintf(out,"5:%.6f 6:%.6f ",DF[2][6],DF[2][7]);
             fprintf(out,"7:%.6f 8:%.6f ",DF[2][8],DF[2][10]);
             fprintf(out,"9:%.6f 10:%.6f ",DF[2][13],DF[2][14]);
             fprintf(out,"11:%.6f 12:%.6f ",DF[2][15],DF[2][16]);
             fprintf(out,"13:%.6f 14:%.6f ",DF[2][20],DF[2][26]);
             fprintf(out,"15:%.6f 16:%.6f ",DF[2][28],DF[2][30]);
             fprintf(out,"17:%.6f 18:%.6f ",DF[2][35],DF[2][37]);
             fprintf(out,"19:%.6f 20:%.6f\n",DF[2][41],DF[2][45]); break;
    case 10: fprintf(out,"1:%.6f 2:%.6f ",TF[2][0],TF[2][14]);
             fprintf(out,"3:%.6f 4:%.6f ",TF[2][18],TF[2][48]);
             fprintf(out,"5:%.6f 6:%.6f ",TF[2][80],TF[2][110]);
             fprintf(out,"7:%.6f 8:%.6f ",TF[2][171],TF[2][182]);
             fprintf(out,"9:%.6f 10:%.6f ",TF[2][186],TF[2][203]);
             fprintf(out,"11:%.6f 12:%.6f ",TF[2][237],TF[2][260]);
             fprintf(out,"13:%.6f 14:%.6f ",TF[2][272],TF[2][295]);
             fprintf(out,"15:%.6f\n",TF[2][330]); break;
    case 11: s=Cluster_Size(3);
             for(i=0;i<s;i++)
             { if(MF[3][i]!=0){ fprintf(out,"%d:%.6f ",i+1,MF[3][i]); } }
             fprintf(out,"\n"); break;
    case 12: fprintf(out,"1:%.6f 2:%.6f ",DF[3][2],DF[3][3]);
             fprintf(out,"3:%.6f 4:%.6f ",DF[3][4],DF[3][6]);
             fprintf(out,"5:%.6f 6:%.6f ",DF[3][10],DF[3][11]);
             fprintf(out,"7:%.6f 8:%.6f ",DF[3][15],DF[3][16]);
             fprintf(out,"9:%.6f 10:%.6f ",DF[3][18],DF[3][19]);
             fprintf(out,"11:%.6f 12:%.6f ",DF[3][25],DF[3][26]);
             fprintf(out,"13:%.6f 14:%.6f ",DF[3][27],DF[3][30]);
             fprintf(out,"15:%.6f 16:%.6f ",DF[3][36],DF[3][38]);
             fprintf(out,"17:%.6f 18:%.6f ",DF[3][39],DF[3][43]);
             fprintf(out,"19:%.6f 20:%.6f ",DF[3][51],DF[3][52]);
             fprintf(out,"21:%.6f 22:%.6f ",DF[3][54],DF[3][56]);
             fprintf(out,"23:%.6f 24:%.6f ",DF[3][57],DF[3][60]);
             fprintf(out,"25:%.6f\n",DF[3][63]); break;
    case 13: s=Cluster_Size(4);
             for(i=0;i<s;i++)
             { if(MF[4][i]!=0){ fprintf(out,"%d:%.6f ",i+1,MF[4][i]); } }
             fprintf(out,"\n"); break;
    case 14: fprintf(out,"1:%.6f 2:%.6f ",DF[4][12],DF[4][15]);
             fprintf(out,"3:%.6f 4:%.6f ",DF[4][22],DF[4][23]);
             fprintf(out,"5:%.6f 6:%.6f ",DF[4][30],DF[4][33]);
             fprintf(out,"7:%.6f 8:%.6f ",DF[4][36],DF[4][72]);
             fprintf(out,"9:%.6f 10:%.6f ",DF[4][100],DF[4][106]);
             fprintf(out,"11:%.6f 12:%.6f ",DF[4][113],DF[4][114]);
             fprintf(out,"13:%.6f 14:%.6f ",DF[4][121],DF[4][125]);
             fprintf(out,"15:%.6f 16:%.6f\n",DF[4][162],DF[4][191]); break;
    case 15: fprintf(out,"1:%.6f 2:%.6f ",TF[4][84],TF[4][230]);
             fprintf(out,"3:%.6f 4:%.6f ",TF[4][264],TF[4][565]);
             fprintf(out,"5:%.6f 6:%.6f ",TF[4][753],TF[4][807]);
             fprintf(out,"7:%.6f 8:%.6f ",TF[4][958],TF[4][1124]);
             fprintf(out,"9:%.6f 10:%.6f ",TF[4][1264],TF[4][1280]);
             fprintf(out,"11:%.6f 12:%.6f ",TF[4][1378],TF[4][1568]);
             fprintf(out,"13:%.6f 14:%.6f ",TF[4][1711],TF[4][1740]);
             fprintf(out,"15:%.6f 16:%.6f ",TF[4][1741],TF[4][1850]);
             fprintf(out,"17:%.6f 18:%.6f ",TF[4][2028],TF[4][2214]);
             fprintf(out,"19:%.6f 20:%.6f ",TF[4][2273],TF[4][2385]);
             fprintf(out,"21:%.6f 22:%.6f\n",TF[4][2530],TF[4][2715]); break;
    case 16: s=Cluster_Size(5);
             for(i=0;i<s;i++)
             { if(MF[5][i]!=0){ fprintf(out,"%d:%.6f ",i+1,MF[5][i]); } }
             fprintf(out,"\n"); break;
    case 17: fprintf(out,"1:%.6f 2:%.6f ",DF[5][17],DF[5][18]);
             fprintf(out,"3:%.6f 4:%.6f ",DF[5][21],DF[5][25]);
             fprintf(out,"5:%.6f 6:%.6f ",DF[5][26],DF[5][28]);
             fprintf(out,"7:%.6f 8:%.6f ",DF[5][31],DF[5][33]);
             fprintf(out,"9:%.6f 10:%.6f ",DF[5][34],DF[5][35]);
             fprintf(out,"11:%.6f 12:%.6f ",DF[5][42],DF[5][105]);
             fprintf(out,"13:%.6f 14:%.6f ",DF[5][106],DF[5][108]);
             fprintf(out,"15:%.6f 16:%.6f ",DF[5][110],DF[5][134]);
             fprintf(out,"17:%.6f 18:%.6f ",DF[5][141],DF[5][142]);
             fprintf(out,"19:%.6f 20:%.6f ",DF[5][144],DF[5][145]);
             fprintf(out,"21:%.6f 22:%.6f ",DF[5][151],DF[5][188]);
             fprintf(out,"23:%.6f 24:%.6f ",DF[5][196],DF[5][210]);
             fprintf(out,"25:%.6f 26:%.6f ",DF[5][219],DF[5][270]);
             fprintf(out,"27:%.6f\n",DF[5][280]); break;
    case 18: fprintf(out,"1:%.6f 2:%.6f ",TF[5][173],TF[5][299]);
             fprintf(out,"3:%.6f 4:%.6f ",TF[5][592],TF[5][616]);
             fprintf(out,"5:%.6f 6:%.6f ",TF[5][816],TF[5][962]);
             fprintf(out,"7:%.6f 8:%.6f ",TF[5][1286],TF[5][1320]);
             fprintf(out,"9:%.6f 10:%.6f ",TF[5][1538],TF[5][1541]);
             fprintf(out,"11:%.6f 12:%.6f ",TF[5][1559],TF[5][1746]);
             fprintf(out,"13:%.6f 14:%.6f ",TF[5][1756],TF[5][1762]);
             fprintf(out,"15:%.6f 16:%.6f ",TF[5][1780],TF[5][1812]);
             fprintf(out,"17:%.6f 18:%.6f ",TF[5][1872],TF[5][1930]);
             fprintf(out,"19:%.6f 20:%.6f ",TF[5][1954],TF[5][2037]);
             fprintf(out,"21:%.6f 22:%.6f ",TF[5][2061],TF[5][2105]);
             fprintf(out,"23:%.6f 24:%.6f ",TF[5][2265],TF[5][2374]);
             fprintf(out,"25:%.6f 26:%.6f ",TF[5][2416],TF[5][2443]);
             fprintf(out,"27:%.6f 28:%.6f ",TF[5][2477],TF[5][2704]);
             fprintf(out,"29:%.6f 30:%.6f ",TF[5][2711],TF[5][2957]);
             fprintf(out,"31:%.6f 32:%.6f ",TF[5][2960],TF[5][2970]);
             fprintf(out,"33:%.6f 34:%.6f ",TF[5][3011],TF[5][3081]);
             fprintf(out,"35:%.6f 36:%.6f ",TF[5][3111],TF[5][3249]);
             fprintf(out,"37:%.6f 38:%.6f ",TF[5][3391],TF[5][3429]);
             fprintf(out,"39:%.6f 40:%.6f ",TF[5][3485],TF[5][4077]);
             fprintf(out,"41:%.6f 42:%.6f\n",TF[5][4116],TF[5][4793]); break;
    case 19: s=Cluster_Size(6);
             for(i=0;i<s;i++)
             { if(MF[6][i]!=0){ fprintf(out,"%d:%.6f ",i+1,MF[6][i]); } }
             fprintf(out,"\n"); break;
    case 20: fprintf(out,"1:%.6f 2:%.6f ",DF[6][2],DF[6][3]);
             fprintf(out,"3:%.6f 4:%.6f ",DF[6][5],DF[6][6]);
             fprintf(out,"5:%.6f 6:%.6f ",DF[6][8],DF[6][9]);
             fprintf(out,"7:%.6f 8:%.6f ",DF[6][10],DF[6][11]);
             fprintf(out,"9:%.6f 10:%.6f ",DF[6][12],DF[6][13]);
             fprintf(out,"11:%.6f 12:%.6f ",DF[6][14],DF[6][15]);
             fprintf(out,"13:%.6f 14:%.6f ",DF[6][16],DF[6][22]);
             fprintf(out,"15:%.6f 16:%.6f ",DF[6][26],DF[6][27]);
             fprintf(out,"17:%.6f 18:%.6f ",DF[6][28],DF[6][30]);
             fprintf(out,"19:%.6f 20:%.6f ",DF[6][32],DF[6][36]);
             fprintf(out,"21:%.6f\n",DF[6][44]); break;
  }
  fclose(out);
}

int AA_Index_PHC7(char c)
{
  switch(c)
  { case 'A': return 0; case 'C': return 1; case 'D': return 2;
    case 'E': return 2; case 'F': return 3; case 'G': return 0;
    case 'H': return 4; case 'I': return 0; case 'K': return 4;
    case 'L': return 0; case 'M': return 1; case 'N': return 5;
    case 'P': return 0; case 'Q': return 5; case 'R': return 4;
    case 'S': return 6; case 'T': return 6; case 'V': return 0;
    case 'W': return 3; case 'Y': return 3; }
  return -1;
}

int AA_Index_PS17(char c)
{
  switch(c)
  { case 'A': return 0;  case 'C': return 1;  case 'D': return 2;
    case 'E': return 2;  case 'F': return 3;  case 'G': return 4;
    case 'H': return 5;  case 'I': return 6;  case 'K': return 7;
    case 'L': return 6;  case 'M': return 8;  case 'N': return 9;
    case 'P': return 10; case 'Q': return 9;  case 'R': return 11;
    case 'S': return 12; case 'T': return 13; case 'V': return 14;
    case 'W': return 15; case 'Y': return 16; }
  return -1;
}

int AA_Index_PS14(char c)
{
  switch(c)
  { case 'A': return 0;  case 'C': return 1;  case 'D': return 2;
    case 'E': return 2;  case 'F': return 3;  case 'G': return 4;
    case 'H': return 0;  case 'I': return 5;  case 'K': return 6;
    case 'L': return 7;  case 'M': return 8;  case 'N': return 9;
    case 'P': return 10; case 'Q': return 2;  case 'R': return 11;
    case 'S': return 12; case 'T': return 12; case 'V': return 5;
    case 'W': return 3;  case 'Y': return 13; }
  return -1;
}

int AA_Index_SIT8(char c)
{
  switch(c)
  { case 'A': return 0; case 'C': return 1; case 'D': return 2;
    case 'E': return 2; case 'F': return 3; case 'G': return 0;
    case 'H': return 4; case 'I': return 1; case 'K': return 5;
    case 'L': return 1; case 'M': return 1; case 'N': return 2;
    case 'P': return 6; case 'Q': return 2; case 'R': return 5;
    case 'S': return 7; case 'T': return 7; case 'V': return 1;
    case 'W': return 3; case 'Y': return 3; }
  return -1;
}

int AA_Index_SIT7(char c)
{
  switch(c)
  { case 'A': return 0; case 'C': return 0; case 'D': return 1;
    case 'E': return 0; case 'F': return 2; case 'G': return 3;
    case 'H': return 2; case 'I': return 4; case 'K': return 0;
    case 'L': return 0; case 'M': return 0; case 'N': return 1;
    case 'P': return 5; case 'Q': return 0; case 'R': return 0;
    case 'S': return 6; case 'T': return 4; case 'V': return 4;
    case 'W': return 2; case 'Y': return 2; }
  return -1;
}

int AA_Index_SIT5(char c)
{
  switch(c)
  { case 'A': return 0; case 'C': return 1; case 'D': return 2;
    case 'E': return 2; case 'F': return 1; case 'G': return 0;
    case 'H': return 3; case 'I': return 1; case 'K': return 2;
    case 'L': return 1; case 'M': return 1; case 'N': return 4;
    case 'P': return 3; case 'Q': return 4; case 'R': return 2;
    case 'S': return 4; case 'T': return 4; case 'V': return 1;
    case 'W': return 1; case 'Y': return 4; }
  return -1;
}

int AA_Index_AA20(char c)
{
  switch(c)
  { case 'A': return 0;  case 'C': return 1;  case 'D': return 2;
    case 'E': return 3;  case 'F': return 4;  case 'G': return 5;
    case 'H': return 6;  case 'I': return 7;  case 'K': return 8;
    case 'L': return 9;  case 'M': return 10; case 'N': return 11;
    case 'P': return 12; case 'Q': return 13; case 'R': return 14;
    case 'S': return 15; case 'T': return 16; case 'V': return 17;
    case 'W': return 18; case 'Y': return 19; }
  return -1;
}

int AA_Index(char c, int clus)
{
  if(clus==0){ return AA_Index_AA20(c); }
  if(clus==1){ return AA_Index_SIT5(c); }
  if(clus==2){ return AA_Index_SIT7(c); }
  if(clus==3){ return AA_Index_SIT8(c); }
  if(clus==4){ return AA_Index_PS14(c); }
  if(clus==5){ return AA_Index_PS17(c); }
  if(clus==6){ return AA_Index_PHC7(c); }
  return -1;
}

int Trip_ID(char c1, char c2, char c3, int clus)
{
  int index;
  if(AA_Index(c1,clus)==-1){ return -1; }
  if(AA_Index(c2,clus)==-1){ return -1; }
  if(AA_Index(c3,clus)==-1){ return -1; }
  index = Cluster_Size(clus)*Cluster_Size(clus)*AA_Index(c1,clus);
  index += Cluster_Size(clus)*AA_Index(c2,clus);
  index += AA_Index(c3,clus);
  return index;
}

int Dip_ID(char c1, char c2, int clus)
{
  int index;
  if(AA_Index(c1,clus)==-1){ return -1; }
  if(AA_Index(c2,clus)==-1){ return -1; }
  index = (Cluster_Size(clus)*AA_Index(c1,clus))+AA_Index(c2,clus);
  return index;
}

int Monop_ID(char c, int clus)
{
  return AA_Index(c,clus);
}

void Compute_All_Frequencies()
{
  int i,j,p;
  for(i=0;i<7;i++)
  { for(j=0;j<Len;j++)
    { p=Monop_ID(SEQ[j],i); if(p!=-1){ MF[i][p]++; } }
    for(j=0;j<(Len-1);j++)
    { p=Dip_ID(SEQ[j],SEQ[j+1],i); if(p!=-1){ DF[i][p]++; } }
    for(j=0;j<(Len-2);j++)
    { p=Trip_ID(SEQ[j],SEQ[j+1],SEQ[j+2],i); if(p!=-1){ TF[i][p]++; } } }
  for(i=0;i<7;i++)
  { p=Cluster_Size(i);
    for(j=0;j<p;j++){ MF[i][j]=MF[i][j]/(Len+0.); }
    for(j=0;j<(p*p);j++){ DF[i][j]=DF[i][j]/(Len-1.); }
    for(j=0;j<(p*p*p);j++){ TF[i][j]=TF[i][j]/(Len-2.); } }
}

void Compute_Exposed_Ratio()
{
  int i,n=0;
  for(i=0;i<Len;i++){ if(ACC[i]=='E'){ n++; } }
  ExpR=(n+0.)/(Len+0.);
}

void Compute_Alpha_Ratio()
{
  int i,n=0;
  for(i=0;i<Len;i++){ if(SSS[i]=='H'){ n++; } }
  AlphaR=(n+0.)/(Len+0.);
}

void Compute_Beta_Ratio()
{
  int i,n=0;
  for(i=0;i<Len;i++){ if(SSS[i]=='E'){ n++; } }
  BetaR=(n+0.)/(Len+0.);
}

void Compute_Aliph_Index()
{
  int i;
  for(i=0;i<Len;i++)
  { if(SEQ[i]=='A'){ AliIndex+=1; }
    if(SEQ[i]=='V'){ AliIndex+=2.9; }
    if((SEQ[i]=='I')||(SEQ[i]=='L')){ AliIndex+=3.9; } }
  AliIndex=(AliIndex/(Len+0.))*100.;
}

double AA_Hydrophaty(char c)
{
  switch(c)
  { case 'A': return 1.8;   case 'C': return 2.5;
    case 'D': return -3.5;  case 'E': return -3.5;
    case 'F': return 2.8;   case 'G': return -0.4;
    case 'H': return -3.2;  case 'I': return 4.5;
    case 'K': return -3.9;  case 'L': return 3.8;
    case 'M': return 1.9;   case 'N': return -3.5;
    case 'P': return -1.6;  case 'Q': return -3.5;
    case 'R': return -4.5;  case 'S': return -0.8;
    case 'T': return -0.7;  case 'V': return 4.2;
    case 'W': return -0.9;  case 'Y': return -1.3; }
  return 0;
}

void Compute_Gravy()
{
  int i;
  for(i=0;i<Len;i++){ Gravy+=AA_Hydrophaty(SEQ[i]); }
  Gravy=Gravy/(Len+0.);
}

double AA_MolWeight(char c)
{
  switch(c)
  { case 'A': return 71.0788;  case 'C': return 103.1388;
    case 'D': return 115.0886; case 'E': return 129.1155;
    case 'F': return 147.1766; case 'G': return 57.0519;
    case 'H': return 137.1411; case 'I': return 113.1594;
    case 'K': return 128.1741; case 'L': return 113.1594;
    case 'M': return 131.1926; case 'N': return 114.1038;
    case 'P': return 97.1167;  case 'Q': return 128.1307;
    case 'R': return 156.1875; case 'S': return 87.0782;
    case 'T': return 101.1051; case 'V': return 99.1326;
    case 'W': return 186.2132; case 'Y': return 163.1760; }
  return 0;
}

void Compute_Mol_Weight()
{
  int i;
  for(i=0;i<Len;i++){ MolWeight+=AA_MolWeight(SEQ[i]); }
}

void Compute_Abs_Charge()
{
  int i,n=0;
  for(i=0;i<Len;i++)
  { if((SEQ[i]=='R')||(SEQ[i]=='K')){ n++; }
    if((SEQ[i]=='D')||(SEQ[i]=='E')){ n--; } }
  AbsCharge=((n+0.)/(Len+0.))-0.03;
  if(AbsCharge<0){ AbsCharge*=-1.; }
}

int Is_Turn_Residue(char c)
{
  if((c=='N')||(c=='G')||(c=='P')||(c=='S')){ return 1; }
  return 0;
}

void Compute_Turn_Forming()
{
  int i,n=0;
  for(i=0;i<Len;i++){ if(Is_Turn_Residue(SEQ[i])){ n++; } }
  TurnForm=(n+0.)/(Len+0.);
}

void Read_Input_File()
{
  int i; char c;
  if((in=fopen(FileIN,"r"))==NULL){ exit(0); }
  
  /** Primary sequence **/
  i=0; c=fgetc(in);
  while((c!='\n')&&(c!=EOF))
  { if(!Is_Sep(c)){ SEQ[i]=toupper(c); i++; } c=fgetc(in); }
  Clean_Array(SEQ,i,SIZE_SEQ); i=0; c=fgetc(in);
  Len=strlen(SEQ);
  if((Len<10)||(Len>2000)||(c==EOF)){ exit(0);}
  
  /** Secondary sequence **/
  while((c!='\n')&&(c!=EOF))
  { if(!Is_Sep(c)){ SSS[i]=toupper(c); i++; } c=fgetc(in); }
  Clean_Array(SSS,i,SIZE_SEQ); i=0; c=fgetc(in);
  if((strlen(SSS)!=Len)||(c==EOF)){ exit(0);}
  
  /** Solvant accessibility **/
  while((c!='\n')&&(c!=EOF))
  { if(!Is_Sep(c)){ ACC[i]=toupper(c); i++; } c=fgetc(in); }
  Clean_Array(ACC,i,SIZE_SEQ);
  if((strlen(ACC)!=Len)||(c==EOF)){ exit(0);}
  
  /** Domains **/
  fscanf(in,"%d",&NbDom);
  if((NbDom<1)||(NbDom>6)){ exit(0); }
  
  fclose(in);
}

void Check_Script_Parameter()
{
  if((FileIN[0]!='/')||(FileIN[1]!='h')){ exit(0); }
}

void Initialize_Protein_Features()
{ 
  int i,j;
  for(i=0;i<SIZE_SEQ;i++)
  { SEQ[i]='\0'; SSS[i]='\0'; ACC[i]='\0';}
  NbDom=0; Len=0; TurnForm=0; AbsCharge=0; MolWeight=0;
  Gravy=0; AliIndex=0; BetaR=0; AlphaR=0; ExpR=0;
  for(i=0;i<7;i++)
  { for(j=0;j<20;j++){ MF[i][j]=0; }
    for(j=0;j<400;j++){ DF[i][j]=0; }
    for(j=0;j<8000;j++){ TF[i][j]=0; } }
}
