#! /usr/bin/perl -w

##################################################################
#
# Predict solubility for one single sequence in fasta format
#
# Input:
#   - predict_ssa.sh
#   - predict_acc.sh
#   - predict_dom.sh
#   - bin directory
#   - models directory
#   - tmp directory
#   - fasta file
#   - output file
#
# Author: Christophe N. Magnan, April 14, 2009
#
##################################################################

# load parameters
if (@ARGV != 8){ die "need 8 parameters...\n"; }
$sspro_sh    = shift @ARGV;
$accpro_sh   = shift @ARGV;
$dompro_sh   = shift @ARGV;
$bin_dir     = shift @ARGV;
$models_dir  = shift @ARGV;
$tmp_dir     = shift @ARGV;
$input_file  = shift @ARGV;
$output_file = shift @ARGV;

# check if the required external scripts exist
if(! -f "$sspro_sh"){ die "SSpro 4.0 is not correctly installed\n"; }
if(! -f "$accpro_sh"){ die "SSpro 4.0 is not correctly installed\n"; }
if(! -f "$dompro_sh"){ die "DOMpro 1.0 is not correctly installed\n"; }

# check the directories in the package
if(substr($bin_dir,length($bin_dir)-1,1) ne "/"){ $bin_dir .= "/";  }
if(substr($models_dir,length($models_dir)-1,1) ne "/"){ $models_dir .= "/";  }
if(substr($tmp_dir,length($tmp_dir)-1,1) ne "/"){ $tmp_dir .= "/";  }
if(! -d $bin_dir || ! -d $models_dir || ! -d $tmp_dir)
{ die "Some subdirectories of SOLpro don't exist. Check the installation tar ball.\n"; }

# check the files in the package
$features1 = "${bin_dir}features1";
$features2 = "${bin_dir}features2";
$svm_scale = "${bin_dir}svm-scale";
$svm_predict = "${bin_dir}svm-predict";
if(! -f $features1 || ! -f $features2 || ! -f $svm_scale || ! -f $svm_predict)
{ die "Please execute configure.pl to install SOLpro correctly.\n"; }
@feat = ("01","02","03","04","05","06","07","08","09","10","11","12","13","14","15","16","17","18","19","20","21");
for $i (0..$#feat)
{ if(! -f "${models_dir}FeatSet$feat[$i].model" || ! -f "${models_dir}FeatSet$feat[$i].range")
  { die "Some files in the solpro package are not found. Please check the installation tar ball.\n"; } }

# read fasta file
open(INPUT,"<$input_file") || die "Can't open fasta file.\n";
$sequence_id = <INPUT>; chomp $sequence_id;
$sequence = <INPUT>;    chomp $sequence;
close INPUT;
$sequence =~ s/\s//sg;
$sequence = uc($sequence);

# check fasta file
if(substr($sequence_id,0,1) ne ">")
{ die "The input file is not in fasta file format.\n"; }
if(length($sequence) < 10 || length($sequence) > 2000)
{ die "The sequence length must be in the range [10:2000]\n."; }

# unique identifier for the temporary files
($sec,$min,$hou,$day,$mon,$yea,$tmp,$tmp,$tmp) = localtime(time);
if($mon<9){ $mon="0".($mon+1); } else{ $mon=$mon+1; }
if($day<10){ $day="0".$day; } if($hou<10){ $hou="0".$hou; }
if($min<10){ $min="0".$min; } if($sec<10){ $sec="0".$sec; }
$job_id =  (1900+$yea).".".$mon.".".$day."-".$hou."h".$min."m".$sec."s-".(int(rand(1000000))%100);
$tmpf = $tmp_dir.$job_id;

# create temporary fasta file
open(TMP_FASTA,">${tmpf}.fa") or die "Couldn't write temporary files\n.";
print TMP_FASTA "$sequence_id\n$sequence\n";
close(TMP_FASTA);

# launch SSpro, ACCpro and DOMpro on the sequence
print "Prediction of protein secondary structure...\n";
`$sspro_sh ${tmpf}.fa ${tmpf}.ss`;
print "Prediction of relative solvent accessibility...\n";
`$accpro_sh ${tmpf}.fa ${tmpf}.acc`;
print "Prediction of domains...\n";
`$dompro_sh ${tmpf}.fa ${tmpf}.dom > ${tmpf}.domo`;

# check output files of SSpro, ACCpro and DOMpro
if(! -f "${tmpf}.ss" || ! -f "${tmpf}.acc" || ! -f "${tmpf}.dom" || ! -f "${tmpf}.domo")
{ die "An error occured during the execution of SSpro or ACCpro or DOMpro.\n"; }

# read SSpro, ACCpro and DOMpro predictions
open(TMP_PRED,"<${tmpf}.ss") or die "Couldn't read temporary files\n.";
$tmp = <TMP_PRED>;
$tmp = <TMP_PRED>;
$secondary_structure = <TMP_PRED>;
chomp $secondary_structure;
close TMP_PRED;
open(TMP_PRED,"<${tmpf}.acc") or die "Couldn't read temporary files\n.";
$tmp = <TMP_PRED>;
$tmp = <TMP_PRED>;
$solvent_sccessibility = <TMP_PRED>;
chomp $solvent_sccessibility;
close TMP_PRED;
open(TMP_PRED,"<${tmpf}.domo") or die "Couldn't read temporary files\n.";
$tmp = <TMP_PRED>;
chomp $tmp;
$nb_domains = substr($tmp,length($tmp)-1,1);
close TMP_PRED;

# check SSpro, ACCpro and DOMpro predictions
if(length($sequence) ne length($secondary_structure))
{ die "An error occured during the execution of SSpro.\n"; }
if(length($sequence) ne length($solvent_sccessibility))
{ die "An error occured during the execution of ACCpro.\n"; }
if(! ($nb_domains > 0  &&  $nb_domains < 6))
{ die "An error occured during the execution of DOMpro.\n"; }

# remove temporary files 1
`rm -rf ${tmpf}.fa ${tmpf}.ss ${tmpf}.acc ${tmpf}.dom`;

# write the file needed for features computation
open(TMP_IN,">${tmpf}.fs") or die "Couldn't write temporary files\n.";
print TMP_IN "$sequence\n$secondary_structure\n$solvent_sccessibility\n$nb_domains\n";
close TMP_IN;

# compute features for the first layer classifiers
`$features1 ${tmpf}.fs`;
for $i (0..($#feat-1))
{ if(! -f "${tmpf}.fs$feat[$i]")
  { die "An error occured during the process, please check the installation.\n"; } }

# scale features to the [-1:1] range
for $i (0..($#feat-1))
{ `$svm_scale -r ${models_dir}FeatSet$feat[$i].range ${tmpf}.fs$feat[$i] > ${tmpf}.fs$feat[$i].scale`; }
for $i (0..($#feat-1))
{ if(! -f "${tmpf}.fs$feat[$i].scale")
  { die "An error occured during the process, please check the installation.\n"; } }

# remove temporary files 2
for $i (0..($#feat-1)){ `rm -rf ${tmpf}.fs$feat[$i]`; }

# Predict solubility (first layer classifiers)
for $i (0..($#feat-1))
{ `$svm_predict -b 1 ${tmpf}.fs$feat[$i].scale ${models_dir}FeatSet$feat[$i].model ${tmpf}.fs$feat[$i].pred`; }
for $i (0..($#feat-1))
{ if(! -f "${tmpf}.fs$feat[$i].pred")
  { die "An error occured during the process, please check the installation.\n"; } }

# remove temporary files 3
for $i (0..($#feat-1)){ `rm -rf ${tmpf}.fs$feat[$i].scale`; }

# final features
`$features2 ${tmpf}.fs`;
if(! -f "${tmpf}.fs21")
{ die "An error occured during the process, please check the installation.\n"; }

# remove temporary files 4
for $i (0..($#feat-1)){ `rm -rf ${tmpf}.fs$feat[$i].pred`; }

# scale features to the [-1:1] range
`$svm_scale -r ${models_dir}FeatSet21.range ${tmpf}.fs21 > ${tmpf}.fs21.scale`;
if(! -f "${tmpf}.fs21.scale")
{ die "An error occured during the process, please check the installation.\n"; }

# remove temporary files 5
`rm -rf ${tmpf}.fs ${tmpf}.fs21`;

# Predict solubility (second layer classifier)
`$svm_predict -b 1 ${tmpf}.fs21.scale ${models_dir}FeatSet21.model ${tmpf}.fs21.pred`;
if(! -f "${tmpf}.fs21.pred")
{ die "An error occured during the process, please check the installation.\n"; }

# read prediction results
open(TMP_PRED,"<${tmpf}.fs21.pred") or die "Couldn't read temporary files\n.";
$tmp = <TMP_PRED>;
$prediction = <TMP_PRED>;
chomp $prediction;
close TMP_PRED;

# output file
@pred = split(/ /, $prediction);
open(OUTPUT,">$output_file") or die "Couldn't write output file\n.";
print OUTPUT "Sequence:\n$sequence\n\n";
print OUTPUT "Predicted secondary structure:\n$secondary_structure\n\n";
print OUTPUT "Predicted relative solvent accessibility:\n$solvent_sccessibility\n\n";
open(TMP_PRED,"<${tmpf}.domo") or die "Couldn't read temporary files\n.";
while($tmp = <TMP_PRED>){ chomp $tmp; print OUTPUT "$tmp\n"; }
close TMP_PRED;
print OUTPUT "\nPredicted solubility upon overexpression:\n";
if($pred[0] eq "1")
{ print OUTPUT "SOLUBLE with probability $pred[1]\n\n"; }
else
{ print OUTPUT "INSOLUBLE with probability $pred[2]\n\n"; }
close OUTPUT;

# remove temporary files
`rm -rf ${tmpf}.domo ${tmpf}.fs21.scale ${tmpf}.fs21.pred`;
print "done...\n";

